=== modified file '__init__.py'
--- __init__.py	2006-03-24 14:06:54 +0000
+++ __init__.py	2006-03-24 16:22:11 +0000
@@ -299,9 +299,16 @@
         from cbranch import cbranch
         return cbranch(source, target)
 
+class cmd_branches(bzrlib.commands.Command):
+    """Scan a location for branches <BZRTOOLS>"""
+    takes_args = ["location?"]
+    def run(self, location=None):
+        from branches import branches
+        return branches(location)
+
 commands = [cmd_shelve, cmd_unshelve, cmd_shelf, cmd_clean_tree,
             cmd_graph_ancestry, cmd_fetch_ghosts, cmd_patch, cmd_shell,
-            cmd_branch_history, cmd_zap, cmd_cbranch]
+            cmd_branch_history, cmd_zap, cmd_cbranch, cmd_branches]
 
 
 command_decorators = []

=== added file 'branches.py'
--- branches.py	1970-01-01 00:00:00 +0000
+++ branches.py	2006-03-24 16:22:11 +0000
@@ -0,0 +1,12 @@
+from bzrlib.transport import get_transport
+from bzrtools import iter_branches
+
+def branches(location=None):
+    if location is None:
+        location = '.'
+    t = get_transport(location)
+    if not t.listable():
+        print "Can't list this type of location."
+        return 3
+    for branch in iter_branches(t):
+        print branch.base[len(t.base):].rstrip('/')

=== modified file 'bzrtools.py'
--- bzrtools.py	2006-03-16 05:59:11 +0000
+++ bzrtools.py	2006-03-24 16:22:11 +0000
@@ -25,8 +25,9 @@
 
 import bzrlib
 import bzrlib.errors
-from bzrlib.errors import BzrCommandError
-from bzrlib.bzrdir import BzrDir
+from bzrlib.errors import (BzrCommandError, NotBranchError, NoSuchFile,
+                           UnsupportedFormatError, TransportError)
+from bzrlib.bzrdir import BzrDir, BzrDirFormat
 
 def temp_tree():
     dirname = tempfile.mkdtemp("temp-branch")
@@ -301,6 +302,37 @@
     return new_committer
 
 
+def iter_branches(t):
+    """Iterate through all the branches under a transport"""
+    try:
+        bzrdir = bzrdir_from_transport(t)
+        branch = bzrdir.open_branch()
+        if branch.bzrdir is bzrdir:
+            yield branch
+    except (NotBranchError, UnsupportedFormatError):
+        pass
+    except TransportError, e:
+        pass
+    try:
+        for directory in t.list_dir('.'):
+            if directory == ".bzr":
+                continue
+            subt = t.clone(directory)
+            for branch in iter_branches(subt):
+                yield branch
+    except NoSuchFile:
+        pass
+    except TransportError, e:
+        pass
+
+    
+def bzrdir_from_transport(t):
+    """Open a bzrdir from a transport (not a location)"""
+    format = BzrDirFormat.find_format(t)
+    BzrDir._check_supported(format, False)
+    return format.open(t)
+
+
 def run_tests():
     import doctest
     result = doctest.testmod()

=== modified file 'tests/blackbox.py'
--- tests/blackbox.py	2006-03-22 22:18:19 +0000
+++ tests/blackbox.py	2006-03-24 16:22:11 +0000
@@ -75,6 +75,19 @@
         self.runbzr('checkout --lightweight source checkout')
         self.runbzr('zap checkout')
 
+    def test_branches(self):
+        self.runbzr('init source')
+        self.runbzr('init source/subsource')
+        self.runbzr('checkout --lightweight source checkout')
+        self.runbzr('init checkout/subcheckout')
+        self.runbzr('init checkout/.bzr/subcheckout')
+        out = self.capture('branches')
+        lines = out.split('\n')
+        self.assertIs(True, 'source' in lines)
+        self.assertIs(True, 'source/subsource' in lines)
+        self.assertIs(True, 'checkout/subcheckout' in lines)
+        self.assertIs(True, 'checkout' not in lines)
+        self.assertIs(True, 'checkout/.bzr/subcheckout' not in lines)
 
 def test_suite():
     return makeSuite(TestBzrTools)

